#========================================================================================
# package      DE_EPAGES::Amazon::API::Object::PagerMWSProductItems
# state        public
# base         DE_EPAGES::Presentation::API::Object::Pager
#----------------------------------------------------------------------------------------
# description  Pager to display the list of Items found by AWS itemLookup / itemSearch
#               requests
#========================================================================================
package DE_EPAGES::Amazon::API::Object::PagerMWSProductItems;
use base DE_EPAGES::Presentation::API::Object::Pager;
use strict;

use Algorithm::CheckDigits;
use JSON qw ( from_json );

use DE_EPAGES::Amazon::API::Object::MapperMWSProduct;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Warning qw ( Warning );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );


use XML::LibXML;
use XML::LibXML::XPathContext;

sub new {
    my ( $class, $hInfo ) = @_;

    # its should only possible to create an object if its generated with the AdapterFactory
    TestObject('Remote', $hInfo->{'Remote'});

    my $self = $class->SUPER::new($hInfo);

    $self->execute();
    return $self;
}

#========================================================================================
# function     execute
# state        private
#----------------------------------------------------------------------------------------
# syntax       $Pager->execute;
#----------------------------------------------------------------------------------------
# description  Executes the query on Amazon AWS
#----------------------------------------------------------------------------------------
# return       none.
#========================================================================================
sub execute {
    my $self = shift;
    my $oResponse = _callWebService( $self );

    if ( defined $oResponse ){
        my $Mapper = DE_EPAGES::Amazon::API::Object::MapperMWSProduct->new();

        my $ShopAlias = $self->get('Remote')->{'PortalConfig'}->getSite()->alias;
        my $aDTOs = $Mapper->mapJSONToDTO($oResponse, $ShopAlias, $self->get('Remote')->{'PortalURL'});
        $self->set( {
            'AWSItems' => $aDTOs // [],
            'AWSPages' => 1, # workaround. only one Page is neeeded
            'AWSItemCount' => scalar @{$aDTOs},
        });
    }
    else {
        $self->set( {
            'AWSItems' => [],
            'AWSPages' => 1,
            'AWSItemCount' => 0,
        });
    }
}

sub _callWebService {
    my $self = shift;

    my $hParameters = $self->get('Parameters');

    # using || instead of //, because SearchString may be empty strings. In this case we want to use ProductName.
    my $SearchString = $self->get('UriParameters')->{'SearchString'} || $hParameters->{'ProductName'};
    my $BrandName = defined $self->get('UriParameters')->{'BrandName'} ? $self->get('UriParameters')->{'BrandName'} : $hParameters->{'BrandName'};
    my $EanAsin = defined $self->get('UriParameters')->{'EanAsin'} ? $self->get('UriParameters')->{'EanAsin'} : $hParameters->{'UPCEAN'};

    my $Response;
    # set default value to 0 in case $EanAsin is undef
    my $NumberOfResults = 0;
    if (defined $EanAsin) {

        my $IdType = _GetIdType($EanAsin);
        $Response = $self->remote->GetMatchingProduct(
            'ASINs' => $EanAsin,
            'IdType' => $IdType,
        );
        eval {
            $NumberOfResults = from_json($Response)->{'numberOfResults'};
        };
        if (ExistsError()) {
            my $Error = GetError();
            Warning($Error);
        }
    }

    $self->set( {
        'FoundEanAsin' => defined($EanAsin) && $EanAsin ne '' && $NumberOfResults == 0 ? 0 : 1
    });

    # If no valid EAN/ASIN is given proceed and go to keyword search $self->remote->ListMatchingProducts
    return $Response unless $NumberOfResults == 0;

    my @Keywords = grep ({/.{2,}/} split /\s+/,$SearchString);
    return if ( !( scalar @Keywords > 0 ) );

    return $self->remote->ListMatchingProducts(
        'Query'      => join( ',', @Keywords ),
        'BrandName' => $BrandName,
        'ItemPage' => 1
    );
}

#========================================================================================
# function     countItems
# state        public
#----------------------------------------------------------------------------------------
# syntax       $Items = $Pager->countItems;
#----------------------------------------------------------------------------------------
# description  Returns the total number of child objects.
#----------------------------------------------------------------------------------------
# return       $Items | total number of items | integer
#========================================================================================
sub countItems {
    my $self = shift;
    return $self->get('AWSItemCount') // 0;
}

sub countPages {
    my $self = shift;
    return $self->get('AWSPages') // 0;
}

#========================================================================================
# function     items
# state        public
#----------------------------------------------------------------------------------------
# syntax       $aItems = $Pager->items();
#----------------------------------------------------------------------------------------
# description  Returns all objects of one page.
#----------------------------------------------------------------------------------------
# return       $aItems | list of objects | ref.array.object
#========================================================================================
sub items {
    my $self = shift;
    return $self->get('AWSItems') // [];
}

#========================================================================================
# function     remote
# state        public
#----------------------------------------------------------------------------------------
# syntax       $self->remote( );
#----------------------------------------------------------------------------------------
# description  Returns remote adapter
#----------------------------------------------------------------------------------------
# return       $Remote | remote adapter | DE_EPAGES::Amazon::API::Object::RemoteProductAdvertising
#========================================================================================
sub remote {
    my $self = shift;
    return $self->get('Remote');
}

sub _GetIdType {
    my ($ProductNumber) = @_;

    my $IdType = undef;

    my $isbn10 = CheckDigits('isbn');
    my $isbn13 = CheckDigits('isbn13');
    my $ean    = CheckDigits('ean');
    my $upc    = CheckDigits('upc');

    if ($isbn10->is_valid($ProductNumber) or $isbn13->is_valid($ProductNumber)) {
        $IdType = 'ISBN';
    }
    if ($upc->is_valid($ProductNumber)) {
        $IdType = 'UPC';
    }
    if ($ProductNumber =~ /^B0[0-9A-Z]{8}$/) {
        $IdType = 'ASIN';
    }
    if ($ean->is_valid($ProductNumber)) {
        $IdType = 'EAN';
    }

    return $IdType;
}

1;
