#========================================================================================
# package      DE_EPAGES::AppStore::API::Object::ScriptTagAppClient
# base         DE_EPAGES::AppStore::API::Object::DeveloperAppsClient
# state        public
# description  Client for OAuth access token operations for the script-tag ressource.
#========================================================================================
package DE_EPAGES::AppStore::API::Object::ScriptTagAppClient;
use base DE_EPAGES::AppStore::API::Object::DeveloperAppsClient;

use strict;
use DE_EPAGES::AppStore::API::Constants qw (
    SHOPS_RESOURCE_PATH
    SCRIPT_TAGS_RESOURCE_PATH
    MY_SCRIPT_TAGS_URL_QUERY
    UNITY_SCRIPT_TAGS_APP_NAME
    UNITY_SCRIPT_TAGS_APP_SCOPES
);
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );
use DE_EPAGES::EPagesJ::API::Object::UriTemplate;
use DE_EPAGES::Object::API::Factory qw ( LoadRootObject );
use Readonly;

Readonly my $SCRIPT_TAGS_PATH => SHOPS_RESOURCE_PATH. '/{shopAlias}/' . SCRIPT_TAGS_RESOURCE_PATH;
Readonly my $MY_SCRIPT_TAGS_PATH => SHOPS_RESOURCE_PATH. '/{shopAlias}/' . SCRIPT_TAGS_RESOURCE_PATH . MY_SCRIPT_TAGS_URL_QUERY;
Readonly my $SCRIPT_TAG_PATH => SHOPS_RESOURCE_PATH. '/{shopAlias}/' . SCRIPT_TAGS_RESOURCE_PATH . '/{scriptTagID}';

sub ErrorDefinition {
    return {
        'NoAppID' => 'App \'#Name\' is not installed',
    };
}

#========================================================================================
# function     new
# state        public
#----------------------------------------------------------------------------------------
# syntax       DE_EPAGES::AppStore::API::Object::ScriptTagAppClient->new( %args );
#----------------------------------------------------------------------------------------
# description  Instanciates a new ScriptTagAppClient.
#----------------------------------------------------------------------------------------
# input        %args | Input arguments. Possible keys.<ul>
#                 <li>Shop - shop object</li>
#                 <li>ScriptTagsUriTemplate - </li>
#                 <li>ScriptTagTemplate - </li>
#                 <li>MyScriptTagsUriTemplate - ... - DE_EPAGES::EPagesJ::API::Object::UriTemplate</li>
#                 <li>LogCategory - category for commlogger. Defaults to "OAuth" - string</li>
#               </ul> | hash
# return       OAuthTokenClient | new instance of OAuthTokenClient | object
#========================================================================================
sub new {
    my $class = shift;
    my %args  = @_;

    foreach my $Mandatory (qw ( Shop ScriptTagsUriTemplate MyScriptTagsUriTemplate ScriptTagTemplate )) {
        TestObject( $Mandatory, $args{$Mandatory} );
    }
    $args{AppID} = $args{Shop}->get('UnityScriptTagsAppGUID');
    $args{RequestHeaders} = undef;

    return $class->SUPER::new(%args);
}


#========================================================================================
# function     create
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptTagAppClient = DE_EPAGES::AppStore::API::Object::ScriptTagAppClient->create($hArgs);
#----------------------------------------------------------------------------------------
# description  Factory method for a ScriptTagAppClient object.
#               Initializes based on the current configuration.
#----------------------------------------------------------------------------------------
# return       $ScriptTagAppClient | ScriptTagAppClient object | DE_EPAGES::AppStore::API::Object::ScriptTagAppClient
#========================================================================================
sub create {
    my $class = shift;
    my ($hArgs) = @_;

    my $System = LoadRootObject;
    $hArgs->{BaseURL} //= $System->get('OAuthServiceBaseUrl') // $System->get('EPagesJBaseURL');
    $hArgs->{ScriptTagsUriTemplate} //= DE_EPAGES::EPagesJ::API::Object::UriTemplate->new( $hArgs->{BaseURL}, $SCRIPT_TAGS_PATH);
    $hArgs->{MyScriptTagsUriTemplate} //= DE_EPAGES::EPagesJ::API::Object::UriTemplate->new( $hArgs->{BaseURL}, $MY_SCRIPT_TAGS_PATH);
    $hArgs->{ScriptTagTemplate}  //= DE_EPAGES::EPagesJ::API::Object::UriTemplate->new( $hArgs->{BaseURL}, $SCRIPT_TAG_PATH);

    return $class->SUPER::create( $hArgs );
}

#========================================================================================
# function     createApp
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptTagApp = $ScriptTagAppClient->createApp();
#----------------------------------------------------------------------------------------
# description  Creates the EpagesUIScriptTags app for the current shop.
#----------------------------------------------------------------------------------------
# return       $ScriptTagsApp | Unity script-tags app | ref.hash
#========================================================================================
sub createApp {
    my $self = shift;

    my $hApp = $self->SUPER::createApp( UNITY_SCRIPT_TAGS_APP_NAME, undef, UNITY_SCRIPT_TAGS_APP_SCOPES );
    $self->_setAppID($hApp->{'privateAppId'});
    return $hApp;
}

#========================================================================================
# function     deleteApp
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptTagAppClient->deleteApp();
#----------------------------------------------------------------------------------------
# description  Deletes the EpagesUIScriptTags app for the current shop.
#========================================================================================
sub deleteApp {
    my $self = shift;

    $self->SUPER::deleteApp( $self->_getAppID() );
    $self->_setAppID(undef);
    $self->{RequestHeaders} = undef;
    return;
}

#========================================================================================
# function     getApp
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptTagApp = $ScriptTagAppClient->getApp();
#----------------------------------------------------------------------------------------
# description  Returns the EpagesUIScriptTags app of the current shop.
#----------------------------------------------------------------------------------------
# return       $ScriptTagsApp | Unity script-tags app | ref.hash
#========================================================================================
sub getApp {
    my $self = shift;
    Error('NoAppID', {Name => UNITY_SCRIPT_TAGS_APP_NAME}) unless defined $self->_getAppID();

    return $self->SUPER::getApp( $self->_getAppID() );
}

#========================================================================================
# function     existsApp
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ExistsApp = $ScriptTagAppClient->existsApp();
#----------------------------------------------------------------------------------------
# description  Returns true, if the EpagesUIScriptTags app is installed for the current
#               shop, false if not.
#----------------------------------------------------------------------------------------
# return       $ExistsApp | exists app | boolean
#========================================================================================
sub existsApp {
    my $self = shift;

    return (defined $self->_getAppID() && defined $self->getApp());
}

#========================================================================================
# function     addScriptTag
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptTagAppClient->addScriptTag($ScriptUrl);
#----------------------------------------------------------------------------------------
# description  Adds a script tag to the current shop.
#----------------------------------------------------------------------------------------
# input        $ScriptUrl | Url of the related script tag | string
#========================================================================================
sub addScriptTag {
    my $self = shift;
    my ($ScriptUrl) = @_;

    return if $self->isScriptTagPresent($ScriptUrl);

    my $hContent = { 'scriptUrl' => $ScriptUrl };
    my $EncodedContent = $self->encodeRequest( $hContent );
    $self->client->post( $self->_scriptTagsUrl, $self->requestHeaders, $EncodedContent );

    return;
}

#========================================================================================
# function     deleteScriptTag
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptTagAppClient->deleteScriptTag($ScriptUrl);
#----------------------------------------------------------------------------------------
# description  Deletes a script tag from the current shop.
#----------------------------------------------------------------------------------------
# input        $ScriptUrl | Url of the related script tag | string
#========================================================================================
sub deleteScriptTag {
    my $self = shift;
    my ($ScriptUrl) = @_;

    my $hScriptTag = $self->getScriptTag($ScriptUrl);
    $self->client->delete( $self->singleScriptTagUrl($hScriptTag->{'scriptTagId'}), $self->requestHeaders )
        if defined $hScriptTag;

    return;
}

#========================================================================================
# function     getScriptTag
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptTagAppClient->getScriptTag($ScriptUrl);
#----------------------------------------------------------------------------------------
# description  Returns the script-tag of the given Url if any.
#----------------------------------------------------------------------------------------
# input        $ScriptUrl | Url of the related script tag | string
# return       $hScriptTag | script-tag | ref.hash
#========================================================================================
sub getScriptTag {
    my $self = shift;
    my ($ScriptUrl) = @_;

    #check if script tag URL is already present
    my $hFoundScriptTag;
    foreach my $hScriptTag (@{ $self->getAllScriptTags() }) {
        if ($hScriptTag->{'scriptUrl'} eq $ScriptUrl) {
            $hFoundScriptTag = $hScriptTag;
            last;
        }
    }

    return $hFoundScriptTag;
}

#========================================================================================
# function     getAllScriptTags
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptTagAppClient->getAllScriptTags();
#----------------------------------------------------------------------------------------
# description  Returns all script-tags related to the app.
#----------------------------------------------------------------------------------------
# return       $hScriptTags | script-tags | ref.array
#========================================================================================
sub getAllScriptTags {
    my $self = shift;

    my $Response = $self->client->get( $self->_myScriptTagsUrl, $self->requestHeaders );
    my $hScriptTags = $self->decodeResponse($Response->decoded_content);

    return $hScriptTags->{'items'};
}

#========================================================================================
# function     getEverySingleScriptTag
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptTagAppClient->getEverySingleScriptTag();
#----------------------------------------------------------------------------------------
# description  Returns ALL, and we mean ALL, script-tags for this app's shop, not just related to the app.
#               Exists because getAllScriptTags only returns app-related script tags.
#----------------------------------------------------------------------------------------
# return       $hScriptTags | script-tags | ref.array
#========================================================================================
sub getEverySingleScriptTag {
    my $self = shift;

    my $Response = $self->client->get( $self->_scriptTagsUrl, $self->requestHeaders );
    my $hScriptTags = $self->decodeResponse($Response->decoded_content);

    return $hScriptTags->{'items'};
}

#========================================================================================
# function     isScriptTagPresent
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ScriptTagAppClient->isScriptTagPresent($ScriptUrl);
#----------------------------------------------------------------------------------------
# description  Check if a given Url has been already added to the script-tags.
#----------------------------------------------------------------------------------------
# input        $ScriptUrl | Url of the related script tag | string
# return       $IsScriptTagPresent | is script-tag prensent | boolean
#========================================================================================
sub isScriptTagPresent {
    my $self = shift;
    my ($ScriptUrl) = @_;

    return defined $self->getScriptTag($ScriptUrl);
}

sub _shop {
    my $self = shift;
    return $self->{Shop};
}

sub _getAppID {
    my $self = shift;
    return $self->{AppID};
}

sub _setAppID {
    my $self = shift;
    my ($AppID) = @_;
    $self->{AppID} = $AppID;
    $self->_shop->set({'UnityScriptTagsAppGUID' => $AppID});
}

sub _scriptTagsUrl {
    my $self = shift;
    return $self->{ScriptTagsUriTemplate}->uri({ shopAlias => $self->_shop->alias});
}

sub _myScriptTagsUrl {
    my $self = shift;
    return $self->{MyScriptTagsUriTemplate}->uri({ shopAlias => $self->_shop->alias});
}

sub singleScriptTagUrl {
    my $self = shift;
    my ($scriptTagID) = @_;
    return $self->{ScriptTagTemplate}->uri( { shopAlias => $self->_shop->alias, scriptTagID => $scriptTagID });
}

sub requestHeaders {
    my $self = shift;

    $self->{RequestHeaders} //= {
        'Authorization' => 'Bearer ' . $self->getApp()->{'token'},
        'User-Agent'    => 'epages6 via ' . UNITY_SCRIPT_TAGS_APP_NAME . ' app'
    };

    return $self->{RequestHeaders};
}


1;
