use strict;
use 5.010; # Needed because we use state vars
use Getopt::Long;
use HTML::Restrict;
use Time::HiRes qw(time);
use DE_EPAGES::Core::API::Error qw(ExistsError GetError);
use DE_EPAGES::Core::API::Script qw(RunScript);
use DE_EPAGES::Core::API::Warning qw(Warning);
use DE_EPAGES::Database::API::Connection qw(RunOnStore);
use DE_EPAGES::Object::API::Factory qw(LoadRootObject LoadObject ExistsObject);
use DE_EPAGES::Product::DAL::Product qw(DAL_GetAllProductIDsAliasOrderedByShopID); ##no critic (RequireCorrectImports)
use DE_EPAGES::ShopMigration::API::ShopCommonData qw(GetActiveShopLanguages);
use DE_EPAGES::WebInterface::API::MessageCenter qw(SynchronizeCache);

RunScript('Sub' => \&Main, 'Message' => 'long');

sub Main {

    local $| = 1;
    my %Options = (output => 1);
    my @GetOpt = qw(help output! storename=s passwd=s shop=s test);
    usage("Unkown parameter specified.\n") unless GetOptions(\%Options, @GetOpt);
    usage() if $Options{help};
    for (qw(storename shop)) {
        usage(sprintf("Missing parameter %s\n", $_)) unless $Options{$_};
    }

    my $Output = $Options{output};
    my $time1 = time;
    RunOnStore(
        Store      => $Options{storename},
        DBPassword => $Options{passwd},
        Sub        => sub {
            my $Shop = findShop($Options{shop});
            my $Count = UnsetMeta($Shop, $Options{test});
            printf "Unset Meta for %s product%s.\n", $Count, $Count == 1 ? '' : 's' if $Output
        },
    );
    my $time2 = time;
    printf "need %.3f seconds\n", $time2 - $time1 if $Output;

    eval {SynchronizeCache();};
    Warning(GetError()) if ExistsError();
}

sub findShop {
    my ($ShopAlias) = @_;

    my $ShopFolder = LoadRootObject()->child('Shops');
    usage(sprintf('No such shop: %s', $ShopAlias)) unless $ShopFolder->existsChild($ShopAlias);
    my $Shop = $ShopFolder->child($ShopAlias);
    usage(sprintf('"%s" is not a Now shop!', $ShopAlias)) unless $Shop->get('IsUnityRegistered');
    return $Shop;
}

sub GetProductsIterator {
    my ($Shop) = @_;

    my $Iterator = DE_EPAGES::Core::API::Iterator->wrap(
        Source => DAL_GetAllProductIDsAliasOrderedByShopID($Shop->id),
        Handle => sub {
            return $_[1]->[0];
        }
    );
    return $Iterator;
}

sub hasHTML {
    my ($Text) = @_;

    state $Processor = HTML::Restrict->new();
    my $Input = $Text;
    my $Plain = $Processor->process($Text);
    $Input =~ s/\s//gs;
    $Plain =~ s/\s//gs;
    return $Input cmp $Plain ? 1 : 0;
}

sub UnsetMeta {
    my ($Shop, $TestMode) = @_;

    my $Count = 0;
    $TestMode //= 0;
    my $ActiveLanguages = GetActiveShopLanguages($Shop);
    my $Iterator = GetProductsIterator($Shop);
    while (my $ProductID = <$Iterator>) {
        next unless (ExistsObject($ProductID));
        my $Product = LoadObject($ProductID);
        printf '%sProduct %s', $TestMode ? '[TEST] ' : '', $Product->alias;
        foreach my $hLanguage (@$ActiveLanguages) {
            my $LanguageID = $hLanguage->{'LanguageID'};
            my $Meta = $Product->getWithParent('Text', $LanguageID);
            if (not defined $Meta or $Meta eq '') {
                printf ' %s [skip/empty]', $hLanguage->{Code2}
            } elsif (! hasHTML($Meta)) {
                printf ' %s [skip/non-html]', $hLanguage->{Code2}
            } else {
                printf ' %s [unset]', $hLanguage->{Code2};
                $Product->set({ Text => '' }, $LanguageID) unless $TestMode;
            }
            $Count++
        }
        printf "\n";
    }
    return $Count;
}

sub usage {
    my ($ErrorNote) = @_;

    print STDERR $ErrorNote . "\n\n" if $ErrorNote;
    print <<END_USAGE;
Description:

Unsets all html valued meta descriptions in a now shop.

Usage:
    perl $0 [options] [flags]

options:
    -passwd      database user password (optional)
    -storename   name of store (uses [storename] if not specified)
    -shop        shop alias
    -(no)output  suppress non-error output

flags:
    -help        show the command line options
    -test        only print out what would be done without actually doing it

Example:
    perl $0 -storename Store -shop DemoShop
END_USAGE
    exit 2;
}



